
/////////////////////////////////////////////////////////////////////////////

typedef unsigned int typeFeux; // 16 cas possibles ( 32 avec long )

// les feux (ou cibles) possibles
enum {E=0, // eteint        les feux (ou cibles) possibles
      Vl=1,   A=2,   S=4,C=8,R=16,   RR=32,   M=64,Cv=128,                  // puissances de 2
      Vlc=256,Ac=512,Sc=1024,Rc=2048,RRc=4096,Mc=8192,    D=16384,X=32768}; // puissances de 2

enum {RIEN=0,CIBLE=1,CIBLE_BIS=2,CIBLE_TER=4}; // les cibles 

enum {MECANIQUE=1,LUMINEUX=2,AVERTISSEMENT=4,SEMAPHORE=8,CARRE=16,VIOLET=32, // des types de signaux
      RALENTISSEMENT=64,RALENTISSEMENT30=64,RAPPEL_RALENTISSEMENT=128,RAPPEL_RALENTISSEMENT30=128,
      RALENTISSEMENT60=256,RAPPEL_RALENTISSEMENT60=512,DISQUE=1024,BAS=2048,BAL=4096};

class Signal {
protected:
  typeFeux feux,
           feuxInit; // TCO
public:
  int no; // numero du signal
  //char* nom; // le nom du signal   OPTION
  //int type; // le type du signal   OPTION
   
  Signal(typeFeux f,int n) { feux=feuxInit=f; no=n; } // constructeur mini
  //Signal(int t,typeFeux f,int n) { feux=feuxInit=f; no=n; } // constructeur 
  //Signal(char* nm,int t,typeFeux f,int n) { nom=nm; type=t; feux=feuxInit=f; no=n; } // constructeur maxi
  
  typeFeux getFeux() { return feux; } // methode d'acces    
  
  virtual int cibles() { return ouvert()?RIEN:CIBLE; } // les cibles a presenter   OPTION
  
  boolean ferme() { return feux&(S|C|Cv); } // methode de test
  boolean ouvert() { return !ferme(); }     // methode de test
  
  virtual void ouvrir()  {} // ouverture manuelle 
  virtual void fermer()  {} // fermeture manuelle 
  virtual void aubiner() { fermer(); } // fermeture automatique par retrosignalisation
  
  void manoeuvrer(); // manoeuvre effective du signal 

  virtual boolean ralentissement30() { return false; } // true si ralentissement a 30
  virtual boolean ralentissement60() { return false; } // true si ralentissement a 60

  virtual void majFeux() {} // mise a jour du signal precedent
  
  virtual Signal* suivant()   { return NULL; } // le signal suivant  
  virtual Signal* precedent() { return NULL; } // le signal precedent   OPTION

  void init() { feux=feuxInit; } // (re)initialisation // TCO
  
  void erreur(int n); // erreur sur les signaux (tranche 300)
  
// methodes utilitaires
  //boolean clignotant() { return (feux&(Rc+RRc)); } // methode utilitaire   OPTION
  
  Signal* selonAiguille(Aiguille* a,Signal* s1,Signal* s2) { return a->directe()?s1:s2; } 
protected:  
  void manoeuvreEtMajFeux(); // manoeuvre le signal puis fait la mise a jour du precedent

  byte coul() { // couleur pour le TCO
    switch (feux) {
      case C: return COULEUR_ROUGE; 
      case S: return COULEUR_ROUGE; 
      case A: case R: case RR: case A+RR: return COULEUR_JAUNE; 
      case Vl: return COULEUR_VERTE; 
    }
    return COULEUR_GRISE;
  }
};

//  //  // //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

void Signal::manoeuvrer() { // appele quand un signal change (feux et/ou cible)
  // COMPLETER manoeuvrer le signal (s'il existe ) ( utiliser l'entier feu et l'entier no ( eventuellement l'entier type ))
  // gerer la zone d'arret si elle existe
  if (no>=2) tco(CODE_CARRE,coul(),no-1);  // TCO
  else       tco(CODE_SIGNAL,coul(),no+1); // TCO
}

void Signal::manoeuvreEtMajFeux() { Signal* s;  
  s=precedent(); if (s==NULL) { erreur(302); return ; }
  manoeuvrer(); s->majFeux(); 
}

void Signal::erreur(int n) { 
  tco(CODE_ERREUR,0,n); // TCO
} 

/////////////////////////////////////////////////////////////////////////////

