
import processing.serial.*;

int NO_PORT=2; // n° du port USB/serie itilise pour communiquer avec le gestionnaire (Arduino)

boolean points=false; // affichage des points du train pour mise au point
boolean trains=true; // presence des trains

final int N=5; // grossissement (zoom)
final int B=10; // bordure du tco en pixels
final float R=0.33F; // retrait pour separation des zones

final int L=10; // nb lignes du tco
final int C=20; // nb colonnes tco

boolean grille=true; // grille ou pas
boolean infos=true; // noms des zones, aiguilles et signaux

final int CODE_AIGUILLE=0,CODE_ZONE=1,CODE_SIGNAL=2,CODE_CARRE=3,
          CODE_BOUTON=4,CODE_BOUTON_CLI=5,CODE_ERREUR=6,CODE_INFO=7;
          
final int COULEUR_GRISE=#7F7F7F,COULEUR_ROUGE=#FF0000,COULEUR_BLANCHE=#FFFFFF,COULEUR_JAUNE=#FFFF00,
          COULEUR_VERTE=#00FF00,/*COULEUR_BLANCHE_CLI=#FFFFFF,COULEUR_GRISE_CLI=#7F7F7F,*/COULEUR_ROSE=#F9429E;
          
final int COULEUR_FOND=#D3D3D3,COULEUR_NOIRE=#000000,COULEUR_BLEUE=#0000FF,COULEUR_OR=#FFD700;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void mouseClicked() { int x,y; int l,c; Pave p; String nom; char car; // boutons
  x=mouseX; y=mouseY; c=(x-B)/(9*N); l=(y-B)/(9*N); 
  //println(x+" "+y+"   "+l+" "+c);
  if (l>=L || c>=C) return; // hors grille
  p=tco.paves[c][l];
  if (p==null) return; // pave null
  if (!(p instanceof PaveBouton)) return; // test si bouton
  nom=p.nom(); car=nom.charAt(0);
  if (car=='G') { grille=!grille; return; } // bouton special
  if (car=='I') { infos=!infos; return; } // bouton special
  if (car=='<') { if (((PaveBouton)p).no==-2) train1.inverser(); else train2.inverser(); return; } // bouton special
  portWrite(p.no()+'0'); 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Tco tco; Serial port; Train train1,train2;

void setup() { 
  size(1000,500,P2D); noStroke();
  tco=new Tco(); tco.construire();
  print("Les ports serie : ");
  println(Serial.list());
  println("Le port utilise : "+Serial.list()[NO_PORT]);
  port=new Serial(this,Serial.list()[NO_PORT],9600); 
  port.buffer(2);
  portWrite('X'); // (re)initialisation du programme de gestion
  tco.dessiner();  // dessiner le TCO
  train1=new Train(IMPAIR,tco.z0,2,1); train2=new Train(PAIR,tco.z1,0,1); 
  //port.write('A'); port.write('B'); // occupation z0 et z1   ne marche pas ici
  delay(100); 
  oldTime=millis();
}

long oldTime,newTime; int DELAI=50; // pour la vitesse du train
boolean bsc=false; // pour gestion du train
boolean ini=true; // pour gestion du train

void draw() { 
  tco.dessiner(); // dessiner le TCO
  if (points) { affichagePoints(); return; } // pour tests
  newTime=millis(); 
  if (ini && trains) { portWrite('A'); portWrite('B'); ini=false; delay(100); } // la premiere fois
  if (trains) if ((newTime-oldTime)>=DELAI) { bsc=!bsc; oldTime=newTime; if (bsc) { train1.avancer(); train2.avancer(); }}
  if (trains) { train1.dessiner(COULEUR_BLEUE); train2.dessiner(COULEUR_OR); stroke(0); }
  
  // test si commandes
  if (port.available()>=2) { char x; int code,info=0,numero; // 2 bytes
    x=port.readChar(); code=x>>4; info=x&15; 
    numero=port.readChar();  
    //print(code+" "+info+" "+numero+"   ");
    affichage(code,info,numero); // affichage lisible
    // traitement des codes
    switch (code) {
      case CODE_AIGUILLE:     if (numero<2) tco.aiguilles[numero].manoeuvrer(); break;
      case CODE_ZONE:         if (numero<6) tco.zones[numero].colorer(mefCol(info)); break;
      case CODE_SIGNAL:       if (numero-1<2) tco.signaux[numero-1].feu(mefCol(info)); break; // 1 a n
      case CODE_CARRE:        if (numero-1<6) tco.carres[numero-1].feu(mefCol(info)); break;  // 1 a n
      case CODE_BOUTON:       if (numero<8) tco.itineraires[numero].couleur(mefCol(info),false); break;
      case CODE_BOUTON_CLI:   if (numero<8) tco.itineraires[numero].couleur(mefCol(info),true); break;
      case CODE_ERREUR:       print("ERREUR "+numero); break;
      case CODE_INFO:         print("INFO "+numero); break;
      default: print('?');
    }
  } // fin draw
 
  // clignotement des boutons
  long temps=millis(); int periode=500; // periode : 1/2 seconde
  for (int i=0; i<tco.itineraires.length; i++) { Pave p; //  8  itineraires  qui sont en fait des paves
    p=tco.itineraires[i];
    if (p.cli()) { 
      if (((int)(temps/periode))%2==1) p.couleur(COULEUR_GRISE,true);
      else                             p.couleur(COULEUR_BLANCHE,true);
    }
  }
}

String[] noms={"XA","XB","AX","BX", "YA","YB","AY","BY"};
  
void portWrite(int c) {
  println();
  if (c>='0' && c<='7') print("===> itineraire "+noms[c-'0']); else
  if (c>='A' && c<='F') print("===> occuper Z"+(char)(c-'A'+'0')); else
  if (c>='a' && c<='f') print("===> liberer Z"+(char)(c-'a'+'0')); 
  print(" : ");
  port.write(c);
}

void affichagePoints() { int x,y; // pour tests
  for (Zone z : tco.zones) for (Pave p : z.paves) {
    x=p.col; y=p.lin;
    for (Forme f : p.formes) { if (f.points!=null) for (Point pt : f.points) {
      translate(B,B); // bordure
      scale(N); // zoom
      stroke(#FFFF00);
      strokeWeight(3);  
      point(x*9+pt.x,y*9+pt.y);
      strokeWeight(1);  
      resetMatrix();        
    }}
  }
}

int mefCol(int c) { // transformation des n° en couleur
  switch (c) {
    case 0: return COULEUR_GRISE;
    case 1: return COULEUR_ROUGE;
    case 2: return COULEUR_BLANCHE;
    case 3: return COULEUR_JAUNE;
    case 4: return COULEUR_VERTE;
    case 5: return COULEUR_ROSE;
    default: return(0);
  }
}

void affichage(int code,int info,int numero) { // affichage lisible
  switch(code) {
    case CODE_AIGUILLE:   print('A'); break;
    case CODE_ZONE:       print('Z'); break;
    case CODE_SIGNAL:     print('S'); break;
    case CODE_CARRE:      print('C'); break;
    case CODE_BOUTON:     print('B'); break;
    case CODE_BOUTON_CLI: print('b'); break;
    case CODE_ERREUR:     print('E'); break;
    case CODE_INFO:       print('I'); break;
    default:              print('?');   
  }
  print(numero); 
  if (code==CODE_ERREUR) { println(); return; }
  print('='); 
  if (code==CODE_AIGUILLE) { if (info==1) print('/'); else print('|'); }
  else { 
    switch(info) {
      case 0:   print("gris"); break;
      case 1:   print("rouge"); break;
      case 2:   print("blanc"); break;
      case 3:   print("jaune"); break;
      case 4:   print("vert"); break;
      case 5:   print("rose"); break;
      default:  print('?');
    }  
  }
  print(' ');
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////